///////////////////////////////////////////////
//                                           //
// VRLab Mental Vision Common Functions      //
// Author: Achille Peternier                 //
// Copyright: EPFL-VRLab 2005 (C)            //
//                                           //
///////////////////////////////////////////////
#ifndef _MVISIO_COMMON_H
#include <mvisio.h>
#include <SDL\SDL.h>
#include <exfound.h>

#pragma comment(lib, "exfound.lib")
#pragma comment(lib, "mvisio.lib")
#pragma comment(lib, "sdl.lib")
#pragma comment(lib, "sdlmain.lib")



//////////////
// DEFAULTS //
//////////////

#define LINE_WIDTH 2.0f
#ifndef WIN32
   #define Sleep(x) usleep(x*1000)
   float absf(float x)
   {
         return sqrt(x * x);
   }
#else
  #define absf abs
#endif



/////////////////////
// PSEUDOCONSTANTS //
/////////////////////

int MODULE_SIZEX = 0;
int MODULE_SIZEY = 0;

///////////
// Init: //
///////////
#define INIT_MVISIO(name, argc, argv)\
MVSETTINGS settings;\
settings.setTitle(name);\
settings.load("data/config.txt");\
settings.setMVEPath("data/");\
settings.setTexturePath("data/");\
settings.setFontPath("data/");\
settings.setMVAPath("");\
settings.setUsername("1");\
settings.setPassword("1");\
if (argc >= 2) if (strcmp(argv[1], "-f")==0) settings.setFullScreen(true);\
if (MVISIO::init(&settings) == false) {/*MessageBox(NULL, "Unable to init MVisio", "Error", MB_OK);*/ return 1; }\
MODULE_SIZEX = MVSETTINGS::getSDLWindowX();\
MODULE_SIZEY = MVSETTINGS::getSDLWindowY();


///////////
// Font: //
#define USE_FONT()\
MVTEXTURE *fontTexture = new MVTEXTURE();\
fontTexture->setFilter(false, false);\
fontTexture->load("font.tga");\
MVFONT *font = new MVFONT();\
font->setTexture(fontTexture);\
font->loadKerning("kerning.txt");\
MVTEXTURE *baseTexture = new MVTEXTURE();\
baseTexture->load("blue.tga");


//////////////////
// Quit button: //
#define USE_QUITBUTTON(screenX, screenY) \
MVBUTTON *quitButton = new MVBUTTON();\
quitButton->setFont(font);\
quitButton->setSizes(90, 26);\
quitButton->setCaption("^y0004^x0010Exit module");\
quitButton->setPosition((float)(screenX-96), 6.0f);
#define PASS_QUITBUTTON quitButton->pass()
#define PRESSED_QUITBUTTON(id) id == quitButton->getId() 


//////////////////////
// Background logo: //
#define USE_BACKGROUND(screenX, screenY) \
MVTEXTURE *backgroundTexture = new MVTEXTURE();\
backgroundTexture->load("background.tga");\
backgroundTexture->setFilter(false, false);\
MVBOX *background = new MVBOX();\
background->setTexture(backgroundTexture);\
background->setSizes(screenX, screenY);\
background->setColor(1.0f, 1.0f, 1.0f, 1.0f);\
background->setThickness(0);\
background->setPosition(0.0f, 0.0f);
#define PASS_BACKGROUND background->pass()


/////////////
// Camera: //
#define USE_CAMERA(x, y, z, tx, ty, tz) \
MVCAMERA *camera = new MVCAMERA();\
camera->setPosition(x, y, z);\
camera->setTargetNode(camera);\
camera->setTarget(tx, ty, tz);\
MVNODE *cameraMesh = MVISIO::load("camera.MVE"); \
if (cameraMesh == NULL ) return 1;


////////////
// Light: //
#define USE_LIGHT(x, y, z) \
MVLIGHT *light = new MVLIGHT();\
light->setPosition(x, y, z);\
light->setAmbient(0.2f, 0.2f, 0.2f, 1.0f);\
light->setDiffuse(0.6f, 0.6f, 0.6f, 1.0f);\
light->setSpecular(0.2f, 0.2f, 0.2f, 1.0f);


/////////////////
// Select box: //
#define USE_SELECTBOX() \
MVNODE *selectBox = MVISIO::load("selectBox.MVE"); \
if (selectBox == NULL) return 1;


/////////////////
// Select box: //
#define USE_SELECTTORUS() \
MVNODE *selectTorus = MVISIO::load("selectTorus.MVE"); \
if (selectTorus == NULL) return 1;



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define _MVISIO_COMMON_H
#endif
