///////////////////////////////////////////////
//                                           //
// VRLab Mental Vision Module 9: Animation   //
// Author: Achille Peternier                 //
// Copyright: EPFL-VRLab 2006 (C)            //
//                                           //
///////////////////////////////////////////////
#include "../data/common.h"
#pragma comment(lib, "opengl32.lib")



//////////////
// #DEFINES //
//////////////
#define MODULE_NAME "Animation"
#ifndef WIN32
   #define MAX_PATH 512
#endif


////////////
// GLOBAL //
////////////

float zoom = 70.0f;
#define ALL_FRAMES 120



//////////
// MAIN //
//////////

int main(int argc, char *argv[])
{ 
   /////////////////
   // Init MVisio //
   /////////////////
   INIT_MVISIO(MODULE_NAME, argc, argv);   
   
   // Common interface:
   USE_FONT();  
   USE_BACKGROUND(MODULE_SIZEX, MODULE_SIZEY);
   USE_QUITBUTTON(MODULE_SIZEX, MODULE_SIZEY);    
   USE_CAMERA(0.0f, 32.0f, zoom, 0.0f, 0.0f, 0.0f); 
   USE_LIGHT(0.0f, 100.0f, 100.0f);    
   light->setSpecular(0.6f, 0.6f, 0.6f, 1.0f);   
   
   
   ////////////////////
   // Module interface:
   MVTEXTURE *back = new MVTEXTURE();
   back->load("blue.tga");
   
   MVWINDOW *winSettings = new MVWINDOW();
   winSettings->setFont(font);
   winSettings->setCaption("Animation control");
   winSettings->setSizes(296, 80);
   winSettings->setTexture(back);
   winSettings->setPosition((int) (MODULE_SIZEX/10), (int) (MODULE_SIZEY/10));
   
   MVBUTTON *open = new MVBUTTON();
   open->setFont(font);
   open->setCaption("^x0016Open");   
   open->setSizes(60, 18);
   open->setPosition(64, 5);
   open->link(winSettings);
   
   MVBUTTON *save = new MVBUTTON();
   save->setFont(font);
   save->setCaption("^x0016Save");   
   save->setSizes(60, 18);
   save->setPosition(126, 5);
   save->link(winSettings);
   
   MVBUTTON *reset = new MVBUTTON();
   reset->setFont(font);
   reset->setCaption("^x0008Reset");   
   reset->setSizes(60, 18);
   reset->setPosition(188, 5);
   reset->link(winSettings);   
   
   MVBUTTON *prevFrame = new MVBUTTON();
   prevFrame->setFont(font);
   prevFrame->setCaption("^x0002<");   
   prevFrame->setSizes(18, 18);
   prevFrame->setPosition(5, 38);
   prevFrame->link(winSettings);
   
   MVBUTTON *nextFrame = new MVBUTTON();
   nextFrame->setFont(font);
   nextFrame->setCaption("^x0002>");   
   nextFrame->setSizes(18, 18);
   nextFrame->setPosition(273, 38);
   nextFrame->link(winSettings);  
   
   MVBUTTON *play = new MVBUTTON();
   play->setFont(font);  
   play->setCaption("^x0016Play");
   play->setEventType(MV_BUTTON_SWITCH);
   play->setSizes(60, 18);
   play->setPosition(2, 5);
   play->link(winSettings);   
   
   MVHSCROLLBAR *timeline = new MVHSCROLLBAR();
   ((MVBUTTON *)timeline->getChild(0))->setSizes(8, 24);
   ((MVBUTTON *)timeline->getChild(0))->move(0, -4);
   timeline->setSizeX(236);
   timeline->setMaxValue((float) ALL_FRAMES);
   timeline->setPosition(30, 43);   
   timeline->setValue(0.0f);
   timeline->link(winSettings);
   
   // Rotations:
   MVWINDOW *winRotations = new MVWINDOW();
   winRotations->setFont(font);
   winRotations->setCaption("Animation control");
   winRotations->setSizes(293, 184);
   winRotations->setTexture(back);
   winRotations->setPosition((int) (MODULE_SIZEX/10), (int) (MODULE_SIZEY - MODULE_SIZEY/5));
   
   MVBUTTON *SetKey = new MVBUTTON();
   SetKey->setFont(font);  
   SetKey->setCaption("^x0010Set key");
   SetKey->setSizes(70, 18);
   SetKey->setPosition(218, 5);
   SetKey->link(winRotations);
 
   MVBUTTON *UnsetKey = new MVBUTTON();
   UnsetKey->setFont(font);  
   UnsetKey->setCaption("Remove key");
   UnsetKey->setSizes(70, 18);
   UnsetKey->setPosition(218, 25);
   UnsetKey->link(winRotations);
   
   #define SCROLLMIN -180.0f
   #define SCROLLMAX 180.0f
   MVHSCROLLBAR *scrollX = new MVHSCROLLBAR();
   scrollX->setSizeX(128);
   scrollX->setColor(1.0f, 0.0f, 0.0f, 0.5f);
   scrollX->getCursor()->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   scrollX->setMinValue(SCROLLMIN);
   scrollX->setMaxValue(SCROLLMAX);
   scrollX->setValue(0.0f);
   scrollX->setPosition(5.0f, 23.0f);
   scrollX->link(winRotations);
   
   MVHSCROLLBAR *scrollY = new MVHSCROLLBAR();
   scrollY->setSizeX(128);
   scrollY->setColor(0.0f, 1.0f, 0.0f, 0.5f);
   scrollY->getCursor()->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   scrollY->setMinValue(SCROLLMIN);
   scrollY->setMaxValue(SCROLLMAX);
   scrollY->setValue(0.0f);
   scrollY->setPosition(5.0f, 39.0f);
   scrollY->link(winRotations);
   
   MVHSCROLLBAR *scrollZ = new MVHSCROLLBAR();
   scrollZ->setSizeX(128);
   scrollZ->setColor(0.0f, 0.0f, 1.0f, 0.5f);
   scrollZ->getCursor()->setColor(0.0f, 0.0f, 1.0f, 1.0f);
   scrollZ->setMinValue(SCROLLMIN);
   scrollZ->setMaxValue(SCROLLMAX);
   scrollZ->setValue(0.0f);
   scrollZ->setPosition(5.0f, 55.0f);
   scrollZ->link(winRotations);  
   
   MVOPENGL::cullingOff();
   //MVSETTINGS::forceShadersOff();           
   
   // VHumans:
   MVNODE *lydia = MVISIO::load("lydia.MVE");         
   MVNODE *lydiaBoxed = MVISIO::load("lydiaBoxed.MVE");
   
   camera->setTargetNode(lydia->findByNameInHierarchy("HumanoidRoot"));
   
   MVMATERIAL *matSelected = new MVMATERIAL();
   matSelected->setAmbient(1.0f, 0.0f, 0.0f, 1.0f);
   matSelected->setDiffuse(0.2f, 0.0f, 0.0f, 1.0f);
   matSelected->setSpecular(0.1f, 0.0f, 0.0f, 1.0f);
   
   MVMATERIAL *matKeyframed = new MVMATERIAL();
   matKeyframed->setAmbient(0.0f, 1.0f, 0.0f, 1.0f);
   matKeyframed->setDiffuse(0.0f, 0.2f, 0.0f, 1.0f);
   matKeyframed->setSpecular(0.0f, 0.1f, 0.0f, 1.0f);   
   
   MVMATERIAL *oldMaterial = ((MVMESH *)lydiaBoxed->findByNameInHierarchy("sacroiliac"))->getMaterial();
   
   // Animation:
   MVANIM *anim = new MVANIM();  
   anim->setTranslating(false);
   anim->setNumberOfFrames((int) timeline->getMaxValue());
   anim->acquireStructure((MVMESH *) lydia->findByNameInHierarchy("lydia_LOD0"));     
   
  
  
///////////////
// Mainloop: //
///////////////

   int mouseX = 0, mouseY = 0, oldMouseX = 0, oldMouseY = 0;   
   bool mouseBL = false, mouseBR = false;
   bool oldMouseBL = false, oldMouseBR = false;
   bool done = false;  
   MVID currentSelected = 0, previousSelected = 60000;
   float rotationX = 0.0f, rotationY = 0.0f;
   MVMESH *boxSelected = NULL, *boxPrevious = NULL;
   float oldValue = -1.0f;   
   
   float MSX = (float) MODULE_SIZEX;
   float MSY = (float) MODULE_SIZEY;
   bool forceUpdate = true;    
   
   while (!done)   
	{			
	      // Display:		
			if (timeline->getValue() != oldValue || forceUpdate)
			{
			      forceUpdate = false;
			      anim->apply((int) timeline->getValue(), lydia);
			      anim->apply((int) timeline->getValue(), lydiaBoxed);			     
			      oldValue = timeline->getValue();    			      
         
               for (int c=0; c<anim->getNumberOfBones(); c++)
			      {
			            MVMESH *obj = ((MVMESH *)lydiaBoxed->findByNameInHierarchy(anim->getBoneName(c)));
			            if (anim->getKeyframe((int) oldValue, c)->key)
			                  obj->setMaterial(matKeyframed);
			            else
			                  obj->setMaterial(oldMaterial);
			      }       
			      
			      if (boxSelected)
               {
                     scrollX->setValue(boxSelected->getRotation().x);
                     scrollY->setValue(boxSelected->getRotation().y);
                     scrollZ->setValue(boxSelected->getRotation().z); 
                     boxSelected->setMaterial(matSelected);
               }     		
			}	      
         
	      ////////////////////////
	      // Rendering background:	   
	   	MVISIO::clear(true, true, false);	   		   	
			MVISIO::begin2D();			   
			   PASS_BACKGROUND;					     			  
			MVISIO::end2D();		
			
			
			// Left side:		
			MVOPENGL::resize(0, MSY/17.25f, MSX/2.0f, MSY-MSY/5.55f);     
			MVISIO::begin3D(camera);
			   light->pass();				   
			   lydia->pass();
			MVISIO::end3D();	
			
			// Right:	
			MVOPENGL::resize(MSX/2.0f,	MSY/17.25f, MSX/2.0f, MSY-MSY/5.55f);  			
			
			
			////////////////
			// 3D rendering:     	
			MVISIO::begin3D(camera);
			   light->pass();				   
			   lydiaBoxed->pass();			   
			MVISIO::end3D();	
			if (mouseBL && !oldMouseBL && !mouseBR)					
			      currentSelected = MVISIO::select(mouseX, mouseY);	     

         // Update selected:
         if (currentSelected != previousSelected && currentSelected > 0)
         {
               boxSelected = (MVMESH *)lydiaBoxed->findByIdInHierarchy(currentSelected);
               if (boxSelected)
               {
                     if (boxPrevious)
                     {
                           boxPrevious->setMaterial(oldMaterial);
                           int b = anim->findBoneByName(boxPrevious->getName());
                           if (b != -1)
                                 if (anim->getKeyframe((int) timeline->getValue(), b)->key)
                                       boxPrevious->setMaterial(matKeyframed);
                     }                           
                     boxSelected->setMaterial(matSelected);
                     scrollX->setValue(boxSelected->getRotation().x);
                     scrollY->setValue(boxSelected->getRotation().y);
                     scrollZ->setValue(boxSelected->getRotation().z);
                     boxPrevious = boxSelected;
               }               
               previousSelected = currentSelected;               
         }			
         
         // Axis:
         if (boxSelected)
         {
               MVISIO::begin3D(camera);
               #define AXLINE 15.0f
               glMultMatrixf((float *) &boxSelected->getFinalMatrix());
               glDisable(GL_LIGHTING);
               glDisable(GL_TEXTURE_2D);
               glLineWidth(LINE_WIDTH*2.0);
               glBegin(GL_LINES);                  
                  glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
                  glVertex3f(0.0f, 0.0f, 0.0f);
                  glColor4f(0.6f, 0.0f, 0.0f, 1.0f);
                  glVertex3f(AXLINE, 0.0f, 0.0f);
               glEnd();
               glBegin(GL_LINES);                  
                  glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
                  glVertex3f(0.0f, 0.0f, 0.0f);
                  glColor4f(0.0f, 0.6f, 0.0f, 1.0f);
                  glVertex3f(0.0f, AXLINE, 0.0f);
               glEnd();
               glBegin(GL_LINES);                  
                  glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
                  glVertex3f(0.0f, 0.0f, 0.0f);
                  glColor4f(0.0f, 0.0f, 0.6f, 1.0f);
                  glVertex3f(0.0f, 0.0f, AXLINE);
               glEnd();
               MVISIO::end3D();
         }          			      
			
			
			////////////////////////
			// Rendering foreground:
			MVOPENGL::resize(0, 0, MODULE_SIZEX, MODULE_SIZEY);
			MVISIO::begin2D();	
			   glLineWidth(LINE_WIDTH);	
			   MVOPENGL::blendingOff();
			   MVTEXTURE::texturingOff();		   
			   glDisable(GL_BLEND);
			   glDisable(GL_TEXTURE_2D);
			   glBegin(GL_LINES);			      
			      glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
			      glVertex2i(MODULE_SIZEX/2.0f, MODULE_SIZEY/18.2);				      
			      glVertex2i(MODULE_SIZEX/2.0f, MODULE_SIZEY-MODULE_SIZEY/8.1);
			   glEnd();		 			   
			   
			   // Title:
			   font->putString(10, 13, MODULE_NAME);	 
            
			   // Interface:
			   PASS_QUITBUTTON;	
			   winSettings->pass();	
			   winRotations->pass();				   			  		  			  			   			
			MVISIO::end2D();
			
			
			////////////////
			// Check events:			
			SDL_Event _event;			
			//SDL_PumpEvents();						
			while (SDL_PollEvent(&_event)) 
			{
					// Quit by click on X:
					if (_event.type == SDL_QUIT) 
							done = true;	
					
					// Resize?
				   if (_event.type == SDL_VIDEORESIZE)
				         MVSETTINGS::resize(0, 0, _event.resize.w, _event.resize.h);							
               
               // Quit by pressing ESC:
					if (_event.type == SDL_KEYDOWN) 
					{
							// Quit by ESC:
							if (_event.key.keysym.sym == SDLK_ESCAPE) 
							       done = true;									       
                    			       													
			      }															
			}     
			
			// Update keyboard:    
			Uint8 *_array;
			int arraySize;
			_array = SDL_GetKeyState(&arraySize);
			if (_array[SDLK_PAGEUP])    
			{
               zoom += 200.0f * MVISIO::getFPSfactor();  
               if (zoom > 350.0f)
			            zoom = 350.0f;			             
         }							       								                     
         else
               if (_array[SDLK_PAGEDOWN])
               {
                     zoom -= 200.0f * MVISIO::getFPSfactor();
                     if (zoom < 50.0f)
			                  zoom = 50.0f;                                          
               }		
			
			// Update mouse:
			oldMouseBR = mouseBR;
			oldMouseBL = mouseBL;
			int buttonState = SDL_GetMouseState(&mouseX, &mouseY);		
         if (buttonState&SDL_BUTTON(1))
					mouseBL = true;					
			else 
					mouseBL = false;
         if (buttonState&SDL_BUTTON(2) || buttonState&SDL_BUTTON(3))
					mouseBR = true;					
			else 
					mouseBR = false;					
					
         float deltaX = (float) (mouseX - oldMouseX);
         float deltaY = (float) (mouseY - oldMouseY);		
         oldMouseX = mouseX; oldMouseY = mouseY;		
         
         // Check MVisio events:
			MVID id=0;
			bool used =	MVELEMENT::manageGUIEvents(0, mouseX, mouseY, mouseBL, mouseBR, &id);  
			if (PRESSED_QUITBUTTON(id))
			      done = true;
         
         // Rotate scene:
         if (mouseBL && mouseBR && !used)
			{
			            zoom += deltaY*2.0f;
			            if (zoom > 350.0f) zoom = 350.0f;
			            if (zoom < 50.0f) zoom = 50.0f;			            
			}
			else
			      if (mouseBL && !used)
			      {
			            // Rotate model:
			            rotationX += deltaY/4.0f;
			            rotationY += deltaX/4.0f;
			            if (rotationX > 68.0f) rotationX = 68.0f;
			            if (rotationX < -68.0f) rotationX = -68.0f;	
			            lydia->setRotation(rotationX, rotationY, 0.0f);		
			            lydiaBoxed->setRotation(rotationX, rotationY, 0.0f);
		         }		
		         
         // Buttons:
         if (play->isClicked())
         {
               play->setCaption("^x0016Stop");
               float f = timeline->getValue();
               f = f + 1.0f;
               if (f > timeline->getMaxValue())
                     f = timeline->getMinValue();
               timeline->setValue(f);                     
         }
         else
               play->setCaption("^x0016Play");		         
			
			// Update window:
			winSettings->clear();
			winSettings->write("^y0060^x0114Frame: %.0f", timeline->getValue());
			winSettings->format();
         camera->setPosition(0.0f, 82.0f, zoom); 
         
         winRotations->clear();
         winRotations->write("^y0002");
         if (boxSelected)
               winRotations->write("Selected: %s\n", boxSelected->getName());
         else
               winRotations->write("Selected: [none]\n");               
         winRotations->write("^x0140X: %.2f \n", scrollX->getValue());               
         winRotations->write("^x0140Y: %.2f \n", scrollY->getValue());
         winRotations->write("^x0140Z: %.2f \n", scrollZ->getValue());  
         winRotations->write("^x0011---------------------------------\n");
         if (boxSelected)
         {
               MVMATRIX mat = boxSelected->getFinalMatrix();
               winRotations->write("Bone matrix (OpenGL order):\n^y0005");
               winRotations->write("^x0020%.2f, %.2f, %.2f, %.2f,\n", mat._11, mat._12, mat._13, mat._14);
               winRotations->write("^x0020%.2f, %.2f, %.2f, %.2f,\n", mat._21, mat._22, mat._23, mat._24);
               winRotations->write("^x0020%.2f, %.2f, %.2f, %.2f,\n", mat._31, mat._32, mat._33, mat._34);
               winRotations->write("^x0020%.2f, %.2f, %.2f, %.2f\n", mat._41, mat._42, mat._43, mat._44);
         }               
         winRotations->format();            
         
         // Update rotations:
         if (boxSelected)
         {                           
               boxSelected->setRotation(scrollX->getValue(), scrollY->getValue(), scrollZ->getValue());
               lydia->findByNameInHierarchy(boxSelected->getName())->setRotation(boxSelected->getRotation()); 
               
               int b = anim->findBoneByName(boxSelected->getName());
               if (b != -1)
                     if (anim->getKeyframe((int) timeline->getValue(), b)->key)
                           UnsetKey->setDisplayStatus(true);
                     else
                           UnsetKey->setDisplayStatus(false);
         }         
         
         // Interpolate:
         if (id == SetKey->getId() && boxSelected)
         {
               anim->setFrame((int) timeline->getValue(), anim->findBoneByName(boxSelected->getName()), scrollX->getValue(), scrollY->getValue(), scrollZ->getValue());
               anim->createKeyframes(0, ALL_FRAMES);
               forceUpdate = true;
         }
         
         // Remove key:
         if (id == UnsetKey->getId() && boxSelected)
         {
               int b = anim->findBoneByName(boxSelected->getName());
               if (b != -1)
               {
                     anim->unsetFrame((int) timeline->getValue(), b);                     
                     anim->createKeyframes(0, ALL_FRAMES);
                     forceUpdate = true;
               }                     
         }
			
			// Open:
			if (id == open->getId())
			{
			      char filename[MAX_PATH];
			      EXF_ZEROMEMORY(filename, MAX_PATH);
			      if (exfFileOpenDialog(filename, "MVA Files (*.mva)\0*.mva\0"))
			      {			      
			            anim->load(filename);
			            forceUpdate = true;
			      }
			}
			
			// Save:
			if (id == save->getId())
			{
			      char filename[MAX_PATH];
			      EXF_ZEROMEMORY(filename, MAX_PATH);
			      if (exfFileSaveDialog(filename, "*.mva", "MVA Files (*.mva)\0*.mva\0"))
			      {
			            anim->save(filename);
			            forceUpdate = true;
			      }			      
			}
			
			// Next frame:
			if (id == nextFrame->getId())
			{
			      if (timeline->getValue() < (float) ALL_FRAMES)
			      {
			            timeline->setValue(timeline->getValue()+1.0f);
			      }
			}
			
			// Prev frame:
			if (id == prevFrame->getId())
			{
			      if (timeline->getValue() > 0.0f)
			      {
			            timeline->setValue(timeline->getValue()-1.0f);
			      }
			}
			
			// Reset:
			if (id == reset->getId())
			{
			      anim->reset();
			      forceUpdate = true;
			}			
			
			Sleep(10); // Don't kill mthreading ;)
			MVISIO::swap();					
 	}      
   
   // Done:
   MVISIO::free();
   return 0;
}
